@@ -115,20 +115,20 @@ class BaseParser(ABC):
     def __init__(self, args: argparse.Namespace | str | list | tuple, config: Config):
         parser = self.build_parser()
         if isinstance(args, str):
-            self.args = parser.parse_args(shlex.split(args))
+            self.args, _ = parser.parse_known_args(shlex.split(args))
             print(self.args)
         elif isinstance(args, (list, tuple)):
-            self.args = parser.parse_args(list(args))
+            self.args, _ = parser.parse_known_args(list(args))
         elif isinstance(args, dict):
-            # dict → namespace, 但仍可以 parser.parse_args([]) 来触发默认值
-            ns = parser.parse_args([])
+            # dict → namespace, 但仍可以 parser.parse_known_args([]) 来触发默认值
+            ns, _ = parser.parse_known_args([])
             for k, v in args.items():
                 setattr(ns, k, v)
             self.args = ns
         elif isinstance(args, argparse.Namespace):
             self.args = args
         elif args is None:
-            self.args = parser.parse_args([])
+            self.args, _ = parser.parse_known_args([])
         else:
             raise TypeError(f"Unsupported args type: {type(args)}")
 
