@@ -22,6 +22,8 @@ TODO: need write ut for this module @Zihao
 
 import os
 import json
+import gzip
+import base64
 import warnings
 from collections import defaultdict
 from typing import Iterator, Optional, MutableMapping, Mapping, Any
@@ -115,8 +117,6 @@ class GraphManager:
     A wrapper for networkx graph, the graph is a MultiDiGraph object.
     """
 
-    _root_nodes: list | None = None
-    _leaf_nodes: list | None = None
     _edge_keys_to_exclude: set = {"u_for_edge", "v_for_edge", "key"}
 
     def __init__(self, file_path: str | None = None) -> None:
@@ -153,15 +153,27 @@ class GraphManager:
 
     @property
     def root_nodes(self) -> list:
-        if self._root_nodes is None:
-            self._root_nodes = [node for node in self.graph.nodes if self.graph.in_degree(node) == 0]
-        return self._root_nodes
+        """
+        Get all root nodes (nodes with in-degree = 0).
+
+        Returns:
+            list: List of root node labels
+
+        Note: Computed on every access to ensure correctness when graph is modified.
+        """
+        return [node for node in self.graph.nodes if self.graph.in_degree(node) == 0]
 
     @property
     def leaf_nodes(self) -> list:
-        if self._leaf_nodes is None:
-            self._leaf_nodes = [node for node in self.graph.nodes if self.graph.out_degree(node) == 0]
-        return self._leaf_nodes
+        """
+        Get all leaf nodes (nodes with out-degree = 0).
+
+        Returns:
+            list: List of leaf node labels
+
+        Note: Computed on every access to ensure correctness when graph is modified.
+        """
+        return [node for node in self.graph.nodes if self.graph.out_degree(node) == 0]
 
     def deduplicate_and_reorder_edges(self) -> "GraphManager":
         """
@@ -605,6 +617,91 @@ class GraphManager:
         """load the graph from the file."""
         return cls(file_path)
 
+    @classmethod
+    def from_dict(cls, data: dict) -> "GraphManager":
+        """
+        Create a GraphManager instance from a dictionary.
+
+        Args:
+            data (dict): A dictionary containing graph data in node-link format.
+                Expected keys: 'directed', 'multigraph', 'graph', 'nodes', 'edges'.
+
+        Returns:
+            GraphManager: A new GraphManager instance with the graph loaded from the dict.
+        """
+        instance = cls()
+        instance.graph = nx.readwrite.json_graph.node_link_graph(data, edges="edges")
+        return instance
+
+    @classmethod
+    def from_web_export(cls, web_export: dict[str, str]) -> "GraphManager":
+        """
+        Reconstruct a GraphManager from web export format.
+
+        This method is designed to work seamlessly with the output from export_for_web().
+        It decompresses and merges the skeleton and metadata to reconstruct the full graph.
+
+        Args:
+            web_export (dict): A dictionary containing compressed graph data:
+                - 'skeleton': Compressed graph structure (base64 encoded)
+                - 'node_metadata': Compressed node attributes (base64 encoded)
+                - 'edge_metadata': Compressed edge attributes (base64 encoded)
+
+        Returns:
+            GraphManager: A new GraphManager instance with the full graph reconstructed.
+
+        Example:
+            >>> # On frontend: decompress, send to backend or reconstruct locally
+            >>> # On backend:
+            >>> graph = GraphManager.from_web_export(web_export)
+            >>> # graph is now a fully reconstructed GraphManager instance
+        """
+        def decompress_and_decode(compressed_str: str) -> dict:
+            """Decompress base64 gzipped string to dict"""
+            compressed_bytes = base64.b64decode(compressed_str.encode('ascii'))
+            json_bytes = gzip.decompress(compressed_bytes)
+            json_str = json_bytes.decode('utf-8')
+            return json.loads(json_str)
+
+        # Decompress all parts
+        skeleton = decompress_and_decode(web_export["skeleton"])
+        node_metadata = decompress_and_decode(web_export["node_metadata"])
+        edge_metadata = decompress_and_decode(web_export["edge_metadata"])
+
+        # Create graph instance
+        instance = cls()
+
+        # Determine graph type
+        if skeleton["directed"] and skeleton["multigraph"]:
+            instance.graph = nx.MultiDiGraph()
+        elif skeleton["directed"]:
+            instance.graph = nx.DiGraph()
+        elif skeleton["multigraph"]:
+            instance.graph = nx.MultiGraph()
+        else:
+            instance.graph = nx.Graph()
+
+        # Add nodes with metadata
+        for node_info in skeleton["nodes"]:
+            node_id = node_info["id"]
+            attrs = node_metadata.get(node_id, {})
+            instance.graph.add_node(node_id, **attrs)
+
+        # Add edges with metadata
+        for edge_info in skeleton["edges"]:
+            u = edge_info["source"]
+            v = edge_info["target"]
+            key = edge_info["key"]
+            edge_key = f"{u}_{v}_{key}"
+            attrs = edge_metadata.get(edge_key, {})
+
+            if skeleton["multigraph"]:
+                instance.graph.add_edge(u, v, key=key, **attrs)
+            else:
+                instance.graph.add_edge(u, v, **attrs)
+
+        return instance
+
     def create_vertex(self, label: str, **kwargs: Any) -> Vertex:
         """Create a vertex"""
         return Vertex(label, parser_name=self.__class__.__name__, **kwargs)
@@ -612,3 +709,68 @@ class GraphManager:
     def create_edge(self, u: str, v: str, **kwargs: Any) -> Edge:
         """Create an edge between two vertices"""
         return Edge(u, v, parser_name=self.__class__.__name__, **kwargs)
+
+    def export_for_web(self) -> dict[str, str]:
+        """
+        Export graph for web scenarios with separate skeleton and metadata.
+
+        This method exports the graph in two parts:
+        1. Skeleton: Basic structure (node IDs and edge connections)
+        2. Metadata: Node and edge attributes
+
+        Both parts are JSON serialized, GZIP compressed, and base64 encoded
+        for efficient transmission to web clients.
+
+        Returns:
+            dict: A dictionary containing:
+                - 'skeleton': Compressed graph structure (base64 encoded)
+                - 'node_metadata': Compressed node attributes (base64 encoded)
+                - 'edge_metadata': Compressed edge attributes (base64 encoded)
+
+        Example:
+            >>> graph = GraphManager()
+            >>> # ... build your graph ...
+            >>> web_export = graph.export_for_web()
+            >>> # Send web_export to frontend
+            >>> # Frontend can decompress and merge to reconstruct full graph
+        """
+        # Extract skeleton (structure only)
+        skeleton = {
+            "directed": self.graph.is_directed(),
+            "multigraph": self.graph.is_multigraph(),
+            "nodes": [{"id": node} for node in self.graph.nodes()],
+            "edges": [
+                {
+                    "source": u,
+                    "target": v,
+                    "key": key
+                }
+                for u, v, key in self.graph.edges(keys=True)
+            ]
+        }
+
+        # Extract node metadata
+        node_metadata = {
+            node: {k: v for k, v in data.items()}
+            for node, data in self.graph.nodes(data=True)
+        }
+
+        # Extract edge metadata
+        edge_metadata = {
+            f"{u}_{v}_{key}": {k: v for k, v in data.items()}
+            for u, v, key, data in self.graph.edges(keys=True, data=True)
+        }
+
+        # Compress and encode
+        def compress_and_encode(data: dict) -> str:
+            """Compress dict to gzipped base64 string"""
+            json_str = json.dumps(data, ensure_ascii=False, cls=DualLicenseEncoder)
+            json_bytes = json_str.encode('utf-8')
+            compressed = gzip.compress(json_bytes, compresslevel=9)
+            return base64.b64encode(compressed).decode('ascii')
+
+        return {
+            "skeleton": compress_and_encode(skeleton),
+            "node_metadata": compress_and_encode(node_metadata),
+            "edge_metadata": compress_and_encode(edge_metadata)
+        }
